@@ -2,6 +2,7 @@ from sqlvalidator.grammar.sql import (
     SelectStatement,
     FunctionCall,
     Column,
+    Type,
     Alias,
     String,
     Integer,
@@ -24,6 +25,9 @@ from sqlvalidator.grammar.sql import (
     UsingClause,
     ExceptClause,
     AnalyticsClause,
+    WindowFrameClause,
+    Case,
+    Index,
 )
 from sqlvalidator.grammar.tokeniser import (
     get_tokens_until_one_of,
@@ -167,7 +171,14 @@ class FromStatementParser:
         elif next_token == "'" or next_token == '"' or next_token == "`":
             expression = Table(StringParser.parse(tokens, next_token))
         else:
-            expression = Table(next_token)
+            if next_token == "unnest":
+                next_next_token = next(tokens)
+                assert next_next_token == "("
+                argument_tokens = get_tokens_until_closing_parenthesis(tokens)
+                arguments = ExpressionListParser.parse(iter(argument_tokens))
+                expression = FunctionCall(next_token, *arguments)
+            else:
+                expression = Table(next_token)
 
         next_token = next(tokens, None)
         if next_token is not None and next_token not in Join.VALUES:
@@ -180,7 +191,7 @@ class FromStatementParser:
             expression = Alias(expression, alias, with_as)
             next_token = next(tokens, None)
 
-        if next_token in Join.VALUES:
+        while next_token in Join.VALUES:
             left_expr = expression
             expression_tokens, next_token = get_tokens_until_not_in(
                 tokens, Join.VALUES, first_token=next_token
@@ -190,16 +201,34 @@ class FromStatementParser:
                 tokens, ("on", "using"), first_token=next_token
             )
             right_expr = FromStatementParser.parse(iter(expression_tokens))
-            on = using = None
-            if next_token == "on":
-                expression = ExpressionParser.parse(tokens)
+            on = None
+            using = None
+            on_or_using = next_token
+            expression_tokens, next_token = get_tokens_until_one_of(
+                tokens,
+                Join.VALUES,
+            )
+            if on_or_using == "on":
+                expression = ExpressionParser.parse(iter(expression_tokens))
                 on = OnClause(expression)
-            elif next_token == "using":
-                expressions = ExpressionParser.parse(tokens)
+            elif on_or_using == "using":
+                expressions = ExpressionParser.parse(iter(expression_tokens))
                 using = UsingClause(expressions)
+            else:
+                raise ParsingError("Missing ON or USING for join")
 
             expression = Join(join_type, left_expr, right_expr, on=on, using=using)
 
+            if next_token is not None and next_token not in Join.VALUES:
+                if next_token == "as":
+                    with_as = True
+                    alias = next(tokens)
+                else:
+                    with_as = False
+                    alias = next_token
+                expression = Alias(expression, alias, with_as)
+                next_token = next(tokens, None)
+
         return expression
 
 
@@ -207,7 +236,7 @@ class JoinTypeParser:
     @staticmethod
     def parse(tokens):
         # TODO: assert known join types
-        return "".join(tokens).upper()
+        return " ".join(tokens).upper()
 
 
 class WhereClauseParser:
@@ -323,8 +352,9 @@ class ExpressionParser:
     @staticmethod
     def parse(tokens, is_right_hand=False):
         main_token = next(tokens)
+        next_token = None
 
-        if main_token == "'" or main_token == '"' or main_token == "`":
+        if main_token in String.QUOTES:
             expression = StringParser.parse(tokens, main_token)
         elif main_token.isdigit():
             expression = Integer(main_token)
@@ -332,14 +362,26 @@ class ExpressionParser:
             expression = Boolean(main_token)
         elif main_token in Null.VALUES:
             expression = Null()
+        elif main_token in Type.VALUES:
+            expression = Type(main_token)
         elif main_token == "(":
             argument_tokens = get_tokens_until_closing_parenthesis(tokens)
             arguments = ExpressionListParser.parse(iter(argument_tokens))
             expression = Parenthesis(*arguments)
+        elif main_token == "case":
+            argument_tokens, next_token = get_tokens_until_one_of(tokens, ["end"])
+            assert next_token == "end"
+            next_token = next(tokens, None)
+            expression = CaseParser.parse(iter(argument_tokens))
+        elif main_token == "select":
+            argument_tokens, next_token = get_tokens_until_one_of(tokens, [])
+            next_token = next(tokens, None)
+            expression = SelectStatementParser.parse(iter(argument_tokens))
         else:
             expression = None
 
-        next_token = next(tokens, None)
+        if next_token is None:
+            next_token = next(tokens, None)
 
         # Expressions that need the next_token to be read
         if expression is None:
@@ -348,12 +390,75 @@ class ExpressionParser:
                 arguments = ExpressionListParser.parse(iter(argument_tokens))
                 expression = FunctionCall(main_token, *arguments)
                 next_token = next(tokens, None)
+            elif next_token is not None and next_token == "[":
+                argument_tokens, next_token = get_tokens_until_one_of(
+                    tokens, stop_words=["]"]
+                )
+                arguments = ExpressionListParser.parse(iter(argument_tokens))
+                expression = Index(
+                    Column(main_token), arguments
+                )  # left item will not always be a column
+                next_token = next(tokens, None)
             elif next_token is not None and main_token == "-" and next_token.isdigit():
                 expression = Integer(-int(next_token))
                 next_token = next(tokens, None)
+            elif (
+                main_token in String.PREFIXES
+                and next_token is not None
+                and next_token in String.QUOTES
+            ):
+                expression = StringParser.parse(
+                    tokens, start_quote=next_token, prefix=main_token
+                )
             else:
                 expression = Column(main_token)
 
+        if next_token == "over":
+            opening_parenthesis = next(tokens, None)
+            if opening_parenthesis != "(":
+                raise ParsingError("expected '('")
+
+            argument_tokens = iter(get_tokens_until_closing_parenthesis(tokens))
+            argument_next_token = next(argument_tokens, None)
+            if argument_next_token == "partition":
+                argument_next_token = next(argument_tokens, None)
+                if not argument_next_token or argument_next_token != "by":
+                    raise ParsingError("Missing BY after PARTITION")
+                expression_tokens, argument_next_token = get_tokens_until_one_of(
+                    argument_tokens, ["order", "rows", "range"]
+                )
+                partition_by = ExpressionListParser.parse(iter(expression_tokens))
+            else:
+                partition_by = None
+
+            if argument_next_token == "order":
+                argument_next_token = next(argument_tokens, None)
+                if not argument_next_token or argument_next_token != "by":
+                    raise ParsingError("Missing BY after ORDER")
+                expression_tokens, argument_next_token = get_tokens_until_one_of(
+                    argument_tokens, ["rows", "range"]
+                )
+                order_by = OrderByParser.parse(iter(expression_tokens))
+            else:
+                order_by = None
+
+            if argument_next_token in ("rows", "range"):
+                rows_range = argument_next_token
+                expression_tokens, _ = get_tokens_until_one_of(argument_tokens, [])
+                frame_clause = WindowFrameClause(
+                    rows_range, " ".join(expression_tokens)
+                )
+            else:
+                frame_clause = None
+
+            expression = AnalyticsClause(
+                expression,
+                partition_by=partition_by,
+                order_by=order_by,
+                frame_clause=frame_clause,
+            )
+            next_token = next(tokens, None)
+
         if next_token and next_token in ("+", "-", "*", "/"):
             left_hand = expression
             symbol = next_token
@@ -387,7 +492,11 @@ class ExpressionParser:
             right_hand_right, next_token = ExpressionParser.parse(
                 tokens, is_right_hand=True
             )
-            right_hand = BooleanCondition("and", right_hand_left, right_hand_right,)
+            right_hand = BooleanCondition(
+                "and",
+                right_hand_left,
+                right_hand_right,
+            )
             expression = Condition(expression, symbol, right_hand)
 
         if next_token in BooleanCondition.PREDICATES:
@@ -397,43 +506,6 @@ class ExpressionParser:
             expression = BooleanCondition(symbol, left_hand, right_hand)
             next_token = next(tokens, None)
 
-        if next_token == "over":
-            opening_parenthesis = next(tokens, None)
-            if opening_parenthesis != "(":
-                raise ParsingError("expected '('")
-
-            argument_tokens = iter(get_tokens_until_closing_parenthesis(tokens))
-            argument_next_token = next(argument_tokens, None)
-            if argument_next_token == "partition":
-                argument_next_token = next(argument_tokens, None)
-                if not argument_next_token or argument_next_token != "by":
-                    raise ParsingError("Missing BY after PARTITION")
-                expression_tokens, argument_next_token = get_tokens_until_one_of(
-                    argument_tokens, ["order"]  # todo frame
-                )
-                partition_by = ExpressionListParser.parse(iter(expression_tokens))
-            else:
-                partition_by = None
-
-            if argument_next_token == "order":
-                argument_next_token = next(argument_tokens, None)
-                if not argument_next_token or argument_next_token != "by":
-                    raise ParsingError("Missing BY after ORDER")
-                expression_tokens, argument_next_token = get_tokens_until_one_of(
-                    argument_tokens, []  # todo frame
-                )
-                order_by = OrderByParser.parse(iter(expression_tokens))
-            else:
-                order_by = None
-
-            expression = AnalyticsClause(
-                expression,
-                partition_by=partition_by,
-                order_by=order_by,
-                frame_clause=None,  # TODO
-            )
-            next_token = next(tokens, None)
-
         if next_token == "except":
             opening_parenthesis = next(tokens, None)
             if opening_parenthesis != "(":
@@ -453,7 +525,7 @@ class ExpressionParser:
         ):
             if next_token == "as":
                 with_as = True
-                alias = next(tokens)
+                alias, _ = ExpressionParser.parse(tokens, is_right_hand=True)
             else:
                 with_as = False
                 alias = next_token
@@ -463,10 +535,45 @@ class ExpressionParser:
 
 class StringParser:
     @staticmethod
-    def parse(tokens, start_quote):
+    def parse(tokens, start_quote, prefix=None):
         string_content = next(tokens, None)
-        string_expression = String(string_content, quotes=start_quote)
-        end_quote = next(tokens)
+        if string_content == start_quote:
+            end_quote = string_content
+            string_content = ""
+        else:
+            end_quote = next(tokens)
+
+        string_expression = String(string_content, quotes=start_quote, prefix=prefix)
         if start_quote != end_quote:
             raise ValueError("Did not find ending quote {}".format(start_quote))
         return string_expression
+
+
+class CaseParser:
+    @staticmethod
+    def parse(tokens):
+        next_token = next(tokens)
+        if next_token == "when":
+            expression = None
+        else:
+            expressions_tokens, next_token = get_tokens_until_one_of(
+                tokens, ["when"], first_token=next_token
+            )
+            expression = ExpressionParser.parse(iter(expressions_tokens))
+
+        when_then = []
+        else_expression = None
+        while next_token:
+            if next_token == "when":
+                expressions_tokens, _ = get_tokens_until_one_of(tokens, ["then"])
+                when_expression = ExpressionParser.parse(iter(expressions_tokens))
+                expressions_tokens, next_token = get_tokens_until_one_of(
+                    tokens, ["when", "else"]
+                )
+                then_expression = ExpressionParser.parse(iter(expressions_tokens))
+                when_then.append((when_expression, then_expression))
+            elif next_token == "else":
+                expressions_tokens, next_token = get_tokens_until_one_of(tokens, [])
+                else_expression = ExpressionParser.parse(iter(expressions_tokens))
+
+        return Case(expression, when_then, else_expression)
