@@ -62,16 +62,17 @@ class SelectStatement:
         if self.from_statement:
             if isinstance(self.from_statement, Alias):
                 alias = self.from_statement
-                self.from_statement = self.from_statement.expression
+                from_statement = self.from_statement.expression
             else:
                 alias = None
+                from_statement = self.from_statement
 
-            if isinstance(self.from_statement, Parenthesis):
+            if isinstance(from_statement, Parenthesis):
                 from_str = "(\n{}\n)".format(
-                    self.from_statement.args[0].transform(is_subquery=True)
+                    from_statement.args[0].transform(is_subquery=True)
                 )
             else:
-                from_str = str(self.from_statement)
+                from_str = transform(from_statement)
 
             if alias:
                 from_str = alias.transform(from_str)
@@ -164,6 +165,36 @@ class SelectStatement:
             and self.order_by_clause == other.order_by_clause
             and self.limit_clause == other.limit_clause
             and self.offset_clause == other.offset_clause
+            and self.semi_colon == other.semi_colon
+        )
+
+    def __repr__(self):
+        return """<{}:
+  Expressions: {!r}
+  Select All: {!r} - Select Distinct: {!r} - Select Distinct On: {!r}
+  From: {!r}
+  Where: {!r}
+  Group By: {!r}
+  Having: {!r}
+  Order By: {!r}
+  Limit: {!r}
+  Offset: {!r}
+  Semi Colon: {!r}
+>
+        """.format(
+            self.__class__.__name__,
+            self.expressions,
+            self.select_all,
+            self.select_distinct,
+            self.select_distinct_on,
+            self.from_statement,
+            self.where_clause,
+            self.group_by_clause,
+            self.having_clause,
+            self.order_by_clause,
+            self.limit_clause,
+            self.offset_clause,
+            self.semi_colon,
         )
 
 
@@ -301,7 +332,9 @@ class OrderByClause(Expression):
         return order_by_str
 
     def __repr__(self):
-        return "<OrderByClause: {}>".format(", ".join(map(repr, self.args)),)
+        return "<OrderByClause: {}>".format(
+            ", ".join(map(repr, self.args)),
+        )
 
     def __eq__(self, other):
         return (
@@ -410,9 +443,19 @@ class FunctionCall(Expression):
         self.args = args
 
     def __str__(self):
-        return "{}({})".format(
-            self.function_name.upper(), ", ".join(map(str, self.args))
-        )
+        transformed_args = [transform(arg) for arg in self.args]
+        with_newlines = any("\n" in arg for arg in transformed_args)
+
+        function_str = self.function_name.upper() + "("
+        if with_newlines:
+            function_str += "\n "
+            function_str += (
+                ",\n ".join(arg.replace("\n", "\n ") for arg in transformed_args)
+                + "\n)"
+            )
+        else:
+            function_str += ", ".join(transformed_args) + ")"
+        return function_str
 
     def __repr__(self):
         return "<FunctionCall {} - {}>".format(
@@ -449,14 +492,74 @@ class AnalyticsClause(Expression):
             analytics_str += "\n ORDER BY{}".format(
                 transform(self.order_by).replace("\n", "\n ")
             )
-        # TODO: frame_clause
+        if self.frame_clause:
+            if "\n" in analytics_str:
+                analytics_str += "\n "
+            analytics_str += transform(self.frame_clause)
 
         analytics_str += "\n)" if "\n" in analytics_str else ")"
         return analytics_str
 
+    def __eq__(self, other):
+        return (
+            type(self) == type(other)
+            and self.function == other.function
+            and (
+                (self.partition_by is None and other.partition_by is None)
+                or (
+                    len(self.partition_by) == len(other.partition_by)
+                    and all(
+                        a == o for a, o in zip(self.partition_by, other.partition_by)
+                    )
+                )
+            )
+            and self.order_by == other.order_by
+            and self.frame_clause == other.frame_clause
+        )
+
+    def __repr__(self):
+        return """<{}:
+  Function: {!r}
+  Partition By: {!r}
+  Order By: {!r}
+  Frame Clause: {!r}
+""".format(
+            self.__class__.__name__,
+            self.function,
+            self.partition_by,
+            self.order_by,
+            self.frame_clause,
+        )
+
+
+class WindowFrameClause(Expression):
+    def __init__(self, rows_range, frame):
+        self.rows_range = rows_range
+        self.frame = frame
+
+    def __str__(self):
+        return "{} {}".format(self.rows_range.upper(), self.frame.upper())
+
+    def __repr__(self):
+        return """<{}:
+  Row Range: {!r}
+  Frame: {!r}
+""".format(
+            self.__class__.__name__,
+            self.rows_range,
+            self.frame,
+        )
+
+    def __eq__(self, other):
+        return (
+            type(self) == type(other)
+            and self.rows_range == other.rows_range
+            and self.frame == other.frame
+        )
+
 
 class Column(Expression):
-    KEYWORDS = ("_table_suffix",)
+    KEYWORDS = ("_table_suffix", "nfc", "nfkc", "nfd", "nfkd")
 
     def __str__(self):
         if self.value in self.KEYWORDS:
@@ -470,13 +573,31 @@ class Column(Expression):
         return errors
 
 
-class String(Expression):
-    def __init__(self, value, quotes):
-        super().__init__(value)
-        self.quotes = quotes
+class Type(Expression):
+    VALUES = ("int", "float", "day", "month", "timestamp")
 
     def __str__(self):
-        return "{quotes}{value}{quotes}".format(quotes=self.quotes, value=self.value)
+        return self.value.upper()
+
+
+class String(Expression):
+    QUOTES = ("'", '"', "`")
+    PREFIXES = ("r",)
+
+    def __init__(self, value, quotes, prefix=None):
+        super().__init__(value)
+        self.quotes = quotes
+        self.prefix = prefix
+
+    def __str__(self):
+        return "{prefix}{quotes}{value}{quotes}".format(
+            quotes=self.quotes, value=self.value, prefix=self.prefix or ""
+        )
+
+    def __repr__(self):
+        return "<{}: {!r} quotes={!r} prefix={!r}>".format(
+            self.__class__.__name__, self.value, self.quotes, self.prefix
+        )
 
     @property
     def return_type(self):
@@ -583,10 +704,12 @@ class Alias(Expression):
 
     def transform(self, expression=None):
         expression = expression or self.expression
-        return "{}{}{}".format(expression, " AS " if self.with_as else " ", self.alias,)
+        return "{}{}{}".format(
+            expression, " AS " if self.with_as else " ", transform(self.alias)
+        )
 
     def __repr__(self):
-        return "<Alias: {!r} as={} {}>".format(
+        return "<Alias: {!r} as={} {!r}>".format(
             self.expression, self.with_as, self.alias
         )
 
@@ -604,6 +727,22 @@ class Alias(Expression):
         return errors
 
 
+class Index(Expression):
+    def __init__(self, expression, indices):
+        super().__init__(expression)
+        self.indices = indices
+
+    def __str__(self):
+        return "{}[{}]".format(
+            transform(self.value), ", ".join(map(transform, self.indices))
+        )
+
+    def __repr__(self):
+        return "<Index: {!r} indices={!r}>".format(
+            self.value, ", ".join(map(repr, self.indices))
+        )
+
+
 class ArithmaticOperator(Expression):
     def __init__(self, operator, *args):
         self.operator = operator
@@ -641,7 +780,7 @@ class Table(Expression):
 
 
 class Join(Expression):
-    VALUES = ("join", "inner", "left", "right", "full", "cross")
+    VALUES = ("join", "inner", "left", "right", "full", "cross", "outer")
 
     def __init__(self, join_type, left_from, right_from, on, using):
         self.join_type = join_type
@@ -653,33 +792,73 @@ class Join(Expression):
     def __str__(self):
         right_from = self.right_from
         num_parenthesis = 0
+
+        alias = None
+        if isinstance(right_from, Alias):
+            alias = right_from
+            right_from = right_from.expression
+
         while isinstance(right_from, Parenthesis):
             right_from = right_from.args[0]
             num_parenthesis += 1
 
-        right_element = "\n" + transform(right_from)
-        right_element = right_element.replace("\n", "\n ")
+        if isinstance(right_from, SelectStatement):
+            right_element = right_from.transform(is_subquery=True)
+        else:
+            right_element = transform(right_from)
+
         if num_parenthesis:
+            right_element = "\n" + right_element
+            if not isinstance(right_from, SelectStatement):
+                right_element = right_element.replace("\n", "\n ")
             right_element = " {}{}\n{}".format(
-                "(" * num_parenthesis, right_element, ")" * num_parenthesis,
+                "(" * num_parenthesis,
+                right_element,
+                ")" * num_parenthesis,
+            )
+        else:
+            right_element = " " + right_element
+
+        if alias:
+            right_element = "{}{} {}".format(
+                right_element, " AS" if alias.with_as else "", alias.alias
             )
 
         left_from = self.left_from
+
+        alias = None
+        if isinstance(left_from, Alias):
+            alias = left_from
+            left_from = left_from.expression
+
         num_parenthesis = 0
         while isinstance(left_from, Parenthesis):
             left_from = left_from.args[0]
             num_parenthesis += 1
 
-        left_element = transform(left_from)
+        if isinstance(left_from, SelectStatement):
+            left_element = left_from.transform(is_subquery=True)
+        else:
+            left_element = transform(left_from)
+
         if num_parenthesis:
             left_element = "\n" + left_element
-            left_element = left_element.replace("\n", "\n ")
+            if not isinstance(left_from, SelectStatement):
+                left_element = left_element.replace("\n", "\n ")
             left_element = "{}{}\n{}".format(
-                "(" * num_parenthesis, left_element, ")" * num_parenthesis,
+                "(" * num_parenthesis,
+                left_element,
+                ")" * num_parenthesis,
+            )
+        if alias:
+            left_element = "{}{} {}".format(
+                left_element, " AS" if alias.with_as else "", alias.alias
             )
 
         join_str = "{}\n{}{}\n".format(
-            left_element, self.join_type.upper(), right_element,
+            left_element,
+            self.join_type.upper(),
+            right_element,
         )
         if self.on:
             join_str += "ON {}".format(transform(self.on))
@@ -687,6 +866,32 @@ class Join(Expression):
             join_str += "USING {}".format(transform(self.using))
         return join_str
 
+    def __eq__(self, other):
+        return (
+            type(self) == type(other)
+            and self.join_type == other.join_type
+            and self.left_from == other.left_from
+            and self.right_from == other.right_from
+            and self.on == other.on
+            and self.using == other.using
+        )
+
+    def __repr__(self):
+        return """<{}:
+  Join Type: {!r}
+  Left part: {!r}
+  Right part: {!r}
+  On: {!r}
+  Using {!r}
+""".format(
+            self.__class__.__name__,
+            self.join_type,
+            self.left_from,
+            self.right_from,
+            self.on,
+            self.using,
+        )
+
 
 class OnClause(Expression):
     pass
@@ -771,6 +976,11 @@ class ExceptClause(Expression):
         super().__init__(expression)
         self.args = args
 
+    def __repr__(self):
+        return "<{}: {!r} args: {!r}>".format(
+            self.__class__.__name__, self.value, self.args
+        )
+
     def __str__(self):
         except_str = "{} EXCEPT (".format(transform(self.value))
         if len(self.args) > 1:
@@ -778,3 +988,26 @@ class ExceptClause(Expression):
         elif except_str:
             except_str += transform(self.args[0])
         return except_str + ")"
+
+
+class Case(Expression):
+    def __init__(self, expression, when_then, else_expression):
+        super().__init__(expression)
+        self.when_then = when_then
+        self.else_expression = else_expression
+
+    def __str__(self):
+        case_str = "CASE"
+        if self.value:
+            case_str += " {}".format(transform(self.value))
+
+        case_str += "\n "
+        case_str += "\n ".join(
+            "WHEN {} THEN {}".format(transform(when), transform(then))
+            for when, then in self.when_then
+        )
+
+        if self.else_expression:
+            case_str += "\n ELSE {}".format(transform(self.else_expression))
+        case_str += "\nEND"
+        return case_str
